@@ -53,7 +53,7 @@ import org.json.JSONException;
 import com.openexchange.ajax.requesthandler.AJAXRequestData;
 import com.openexchange.ajax.requesthandler.AJAXRequestResult;
 import com.openexchange.drive.DirectoryMetadata;
-import com.openexchange.drive.DriveSession;
+import com.openexchange.drive.json.internal.DefaultDriveSession;
 import com.openexchange.drive.json.json.JsonDirectoryMetadata;
 import com.openexchange.exception.OXException;
 import com.openexchange.java.Strings;
@@ -68,7 +68,7 @@ import com.openexchange.tools.servlet.AjaxExceptionCodes;
 public class DirectoryMetadataAction extends AbstractDriveAction {
 
     @Override
-    public AJAXRequestResult doPerform(AJAXRequestData requestData, DriveSession session) throws OXException {
+    public AJAXRequestResult doPerform(AJAXRequestData requestData, DefaultDriveSession session) throws OXException {
         /*
          * get parameters
          */
