@@ -55,8 +55,8 @@ import org.json.JSONObject;
 import com.openexchange.ajax.requesthandler.AJAXRequestData;
 import com.openexchange.ajax.requesthandler.AJAXRequestResult;
 import com.openexchange.drive.DriveQuota;
-import com.openexchange.drive.DriveSession;
 import com.openexchange.drive.DriveSettings;
+import com.openexchange.drive.json.internal.DefaultDriveSession;
 import com.openexchange.exception.OXException;
 import com.openexchange.file.storage.Quota;
 import com.openexchange.java.Strings;
@@ -71,7 +71,7 @@ import com.openexchange.tools.servlet.AjaxExceptionCodes;
 public class SettingsAction extends AbstractDriveAction {
 
     @Override
-    public AJAXRequestResult doPerform(AJAXRequestData requestData, DriveSession session) throws OXException {
+    public AJAXRequestResult doPerform(AJAXRequestData requestData, DefaultDriveSession session) throws OXException {
         /*
          * get request data
          */
@@ -108,6 +108,8 @@ public class SettingsAction extends AbstractDriveAction {
                     }
                 }
                 jsonObject.put("serverVersion", settings.getServerVersion());
+                jsonObject.put("supportedApiVersion", settings.getSupportedApiVersion());
+                jsonObject.put("minApiVersion", settings.getMinApiVersion());
             } catch (JSONException e) {
                 throw AjaxExceptionCodes.JSON_ERROR.create(e, e.getMessage());
             }
